#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019 Baidu.com, Inc. All Rights Reserved
#
"""
target.py
Authors: rensike(rensike@baidu.com)
Date:    2019/9/14 下午5:42
"""


class Target(object):
    area = 0

    @property
    def is_small(self):
        return hasattr(self, "area") and self.area < 322

    @property
    def is_middle(self):
        return hasattr(self, "area") and 322 <= self.area < 962

    @property
    def is_large(self):
        return hasattr(self, "area") and self.area >= 962



# -*- coding: UTF-8 -*-
from ..vis.color import VIS_COLOR
from ..utils import is_seq, is_dir, mkdir
from ..image.transforms import imcrop, imresize
from .classify import Classify
from .core import BBox, Image
from abc import ABCMeta, abstractmethod
from collections import OrderedDict
from terminaltables import AsciiTable
import shutil
import random
import copy
import os
from time import time


class IcvDataSet(object):
    __metaclass__ = ABCMeta

    def __init__(self, ids=None, categories=None, keep_no_anno_image=True, one_index=False, build_cat=True):
        self.ids = ids if ids else []
        self.categories = categories if categories else []
        self.keep_no_anno_image = keep_no_anno_image
        self.one_index = one_index
        self.num_classes = len(self.categories)

        if build_cat:
            self._build()

        if not hasattr(self, "color_map") or self.color_map is None or len(self.color_map) == 0:
            self.set_colormap()

    def _build(self):
        if self.one_index:
            self.id2cat = {i + 1: cat for i, cat in enumerate(self.categories)}
            self.cat2id = {cat: i + 1 for i, cat in enumerate(self.categories)}
        else:
            self.id2cat = {i: cat for i, cat in enumerate(self.categories)}
            self.cat2id = {cat: i for i, cat in enumerate(self.categories)}

        self.id2index = {id: i for i, id in enumerate(self.ids)}
        self.index2id = {i: id for i, id in enumerate(self.ids)}

    def get_img_info(self, index):
        img_id = self.ids[index]
        return img_id

    def set_colormap(self, color_list_or_map=None):
        assert color_list_or_map is None or isinstance(color_list_or_map, dict) or is_seq(color_list_or_map)
        if color_list_or_map is not None and len(self.categories) > 0:
            if isinstance(color_list_or_map, dict):
                self.color_map = {c: color_list_or_map[c] for c in color_list_or_map if c in self.categories}
                self.color_map.update({
                    c: random.choice(VIS_COLOR)
                    for c in self.categories if c not in self.color_map
                })
            elif is_seq(color_list_or_map) and len(color_list_or_map) > 0:
                self.color_map = {self.categories[i]: color_list_or_map[i % len(color_list_or_map)] for i in
                                  range(len(self.categories))}
            return
        self.color_map = {}
        _colors = VIS_COLOR
        # random.shuffle(_colors)
        for i, cat in enumerate(self.categories):
            self.color_map[cat] = _colors[i % len(_colors)]

    @property
    def length(self):
        return len(self)

    def __len__(self):
        return len(self.ids)

    def __getitem__(self, item):
        return self.get_sample(self.index2id[item])

    @property
    def is_seg_mode(self):
        if self.length == 0:
            return False

        for i in range(self.length):
            for anno in self.get_sample(self.ids[i]).annos:
                if anno.seg_mode_polys or anno.seg_mode_mask:
                    return True

        return False

    def parse_categories(self):
        categories = []
        for anno_sample in self.samples:
            label_list = [anno.label for anno in anno_sample.annos if anno.label is not None]
            if label_list:
                categories.extend(label_list)
        categories = list(set(categories))
        categories.sort()
        self.set_categories(categories)
        return categories

    def get_categories(self):
        return self.categories

    def set_categories(self, categories):
        assert is_seq(categories)
        self.categories = list(categories)

        self.id2cat = {}
        self.cat2id = {}
        for i, cate in enumerate(self.categories):
            if self.one_index:
                self.id2cat[i + 1] = cate
                self.cat2id[cate] = i + 1
            else:
                self.id2cat[i] = cate
                self.cat2id[cate] = i

    def get_samples(self, ids=None):
        self.samples = []
        ids = ids if ids is not None else self.ids
        for id in ids:
            sample = self.get_sample(id)
            if sample.count == 0 and not self.keep_no_anno_image:
                continue
            self.samples.append(sample)

        return self.samples

    def get_sample(self, id):
        pass

    def get_groundtruth(self, id):
        return self.get_sample(id)

    @abstractmethod
    def vis(self, save_dir=None):
        pass

    @abstractmethod
    def sub(self, count=0, ratio=0, shuffle=True, output_dir=None):
        if count <= 0:
            count = round(self.length * ratio)

        count = min(self.length, count)
        if shuffle:
            ids = random.sample(self.ids, count)
        else:
            ids = self.ids[:count]

        dataset = copy.deepcopy(self)
        dataset.ids = ids
        dataset.categories = dataset.get_categories()

        if output_dir is not None:
            dataset.save(output_dir)

        return dataset

    @abstractmethod
    def save(self, output_dir, reset_dir=False, split=None):
        pass

    @abstractmethod
    def concat(self, dataset, out_dir, reset=False, new_split=None):
        pass

    @abstractmethod
    def crop_bbox_for_classify(self, output_dir, reset=False, split="train", pad=0, resize=None):
        assert resize is None or (is_seq(resize) and len(resize) == 2)
        if reset and is_dir(output_dir):
            shutil.rmtree(output_dir)
        if not is_dir(output_dir):
            mkdir(output_dir)
        samples = self.get_samples()

        tmp_dir = os.path.join(output_dir, str(time()))
        mkdir(tmp_dir)

        classify_dataset = Classify(root_dir=output_dir, categories=self.categories)
        for sample in samples:
            _bboxes = []
            _cats = []
            for anno in sample.annos:
                if isinstance(anno.bbox, BBox):
                    _bbox = anno.bbox.copy()
                    _bbox.extend(pad, sample.shape)
                    _bboxes.append(_bbox.bbox)
                    _cats.append(anno.label)
            patches = imcrop(sample.image, _bboxes)
            assert len(patches) == len(_cats)

            for i, p in enumerate(patches):
                if resize is not None:
                    p = imresize(p, resize)
                imgobj = Image(img=p, ext=sample.ext, name="")
                classify_dataset.img2cat[imgobj] = _cats[i]
                classify_dataset.img2set[imgobj] = split

        classify_dataset.save(output_dir, rename=True)
        shutil.rmtree(tmp_dir)

    @abstractmethod
    def statistic(self, print_log=True, is_plot_show=False, plot_save_path=None):
        RATIO_TOP_K = 15
        samples = self.get_samples()

        sta = dict(overview=dict(), detail=dict())
        sta["overview"] = {
            "image_num": self.length,
            "class_num": len(self.categories),
            "bbox_num": sum([sample.count for sample in samples]),
            "min_width": min([sample.width for sample in samples]),
            "max_width": max([sample.width for sample in samples]),
            "min_height": min([sample.height for sample in samples]),
            "max_height": max([sample.height for sample in samples]),
            "small_bbox_num": 0,
            "middle_bbox_num": 0,
            "large_bbox_num": 0,
            "same_size": len(set([(sample.height, sample.width) for sample in samples])) == 1,
            "ratio_distribution": OrderedDict(),
        }

        sta["detail"] = {
            cat: {
                "num": 0,
                "ratio_distribution": OrderedDict(),
            }
            for cat in self.categories
        }

        # get bbox ratio distribution
        for sample in samples:
            s = sample.statistics()
            for label in s["cats"]:
                sta["detail"][label]["num"] += s["cats"][label]

            for label in s["ratios"]:
                for r in s["ratios"][label]:
                    ratio = str(r)
                    if ratio not in sta["overview"]["ratio_distribution"]:
                        sta["overview"]["ratio_distribution"][ratio] = 0
                    sta["overview"]["ratio_distribution"][ratio] += s["ratios"][label][r]

                    if ratio not in sta["detail"][label]["ratio_distribution"]:
                        sta["detail"][label]["ratio_distribution"][ratio] = 0
                    sta["detail"][label]["ratio_distribution"][ratio] += s["ratios"][label][r]

        # truncate for top k ratio num
        sta["overview"]["ratio_distribution"] = dict(
            OrderedDict(sorted(sta["overview"]["ratio_distribution"].items(), key=lambda t: -t[1])[:RATIO_TOP_K]))

        ratio_kvs = []
        for i, label in enumerate(sta["detail"]):
            tmp_rd = dict(
                OrderedDict(
                    sorted(sta["detail"][label]["ratio_distribution"].items(), key=lambda t: -t[1])[:RATIO_TOP_K * 10]))

            tmp_rd_k = list(tmp_rd.keys())
            tmp_rd_v = list(tmp_rd.values())
            if i == 0:
                ratio_kvs = list(zip(tmp_rd_k, tmp_rd_v))
            else:
                ratio_kvs = [x for x in ratio_kvs if x[0] in tmp_rd_k]

        ratio_kvs = sorted(ratio_kvs, key=lambda t: -t[1])[:RATIO_TOP_K]
        ratio_keys = list(zip(*ratio_kvs))[0]

        for label in sta["detail"]:
            sta["detail"][label]["ratio_distribution"] = {k: sta["detail"][label]["ratio_distribution"][k] for k in
                                                          ratio_keys}

        # get target(bbox/seg) size distribution
        bbox_size_dict = {}
        for sample in samples:
            for anno in sample.annos:
                if anno.label not in bbox_size_dict:
                    bbox_size_dict[anno.label] = []

                if str(round(anno.bbox.ratio, 1)) in ratio_keys:
                    bbox_size_dict[anno.label].append((anno.bbox.width, anno.bbox.height))

                if anno.bbox.is_small:
                    sta["overview"]["small_bbox_num"] += 1
                elif anno.bbox.is_middle:
                    sta["overview"]["middle_bbox_num"] += 1
                elif anno.bbox.is_large:
                    sta["overview"]["large_bbox_num"] += 1

                if self.is_seg_mode:
                    if "small_segarea_num" not in sta["overview"]:
                        sta["overview"]["small_segarea_num"] = 0
                    if "middle_segarea_num" not in sta["overview"]:
                        sta["overview"]["middle_segarea_num"] = 0
                    if "large_segarea_num" not in sta["overview"]:
                        sta["overview"]["large_segarea_num"] = 0

                    if (anno.seg_mode_mask and anno.mask.is_small) or (anno.seg_mode_polys and anno.polys.is_small):
                        sta["overview"]["small_segarea_num"] += 1
                    if (anno.seg_mode_mask and anno.mask.is_middle) or (anno.seg_mode_polys and anno.polys.is_middle):
                        sta["overview"]["middle_segarea_num"] += 1
                    if (anno.seg_mode_mask and anno.mask.is_large) or (anno.seg_mode_polys and anno.polys.is_large):
                        sta["overview"]["large_segarea_num"] += 1

        if print_log or is_plot_show or plot_save_path is not None:
            # for overview
            overview_header = ["image_num", "class_num", "bbox_num", "same_image_size",
                               "min_image_width", "max_image_width", "min_image_height", "max_image_height"]
            overview_header_cn = ["图片总数", "类别总数", "目标框总数", "图片是否同尺寸",
                                  "最小图片宽度", "最大图片宽度", "最小图片高度", "最大图片高度"]
            overview_table_data = [
                sta["overview"]["image_num"], sta["overview"]["class_num"], sta["overview"]["bbox_num"],
                sta["overview"]["same_size"], sta["overview"]["min_width"], sta["overview"]["max_width"],
                sta["overview"]["min_height"], sta["overview"]["max_height"],
            ]
            overview_table = AsciiTable([overview_header, overview_header_cn, overview_table_data])
            overview_table.inner_footing_row_border = True

            # for target statistic
            target_statistic_header = ["bbox_num", "small_bbox_num", "middle_bbox_num", "large_bbox_num"]
            target_statistic_header_cn = ["目标框总数", "小目标框总数", "中目标框总数", "大目标框总数"]
            target_statistic_data = [
                sta["overview"]["bbox_num"],
                sta["overview"]["small_bbox_num"], sta["overview"]["middle_bbox_num"],
                sta["overview"]["large_bbox_num"],
            ]

            if self.is_seg_mode:
                target_statistic_header.extend(["small_segarea_num", "middle_segarea_num", "large_segarea_num"])
                target_statistic_header_cn.extend(["小目标分割区域总数", "中目标分割区域总数", "大目标分割区域总数"])
                target_statistic_data.extend([
                    sta["overview"]["small_segarea_num"], sta["overview"]["middle_segarea_num"],
                    sta["overview"]["large_segarea_num"],
                ])
            target_statistic_table = AsciiTable(
                [target_statistic_header, target_statistic_header_cn, target_statistic_data])
            target_statistic_table.inner_footing_row_border = True

            # for target rartio
            ratios = sorted(list(sta["overview"]["ratio_distribution"].keys()))

            ratio_distri_header = ["bbox ratio (h/w)"] + ratios
            ratio_distri_data = [sta["overview"]["ratio_distribution"][r] for r in ratios]
            ratio_distri_table_data = [
                ratio_distri_header,
                ["-"] + ratio_distri_data
            ]

            ratio_distri_table = AsciiTable(ratio_distri_table_data)
            ratio_distri_table.inner_footing_row_border = True

            # for categories statistic
            cats_detail_header = ["bbox ratio (h/w) \ class"] + self.categories
            cats_detail_table_data = [cats_detail_header]
            cats_num_data = [sta["detail"][cat]["num"] for cat in self.categories]
            cats_detail_table_data.append(["total num"] + cats_num_data)
            cat_ratios = sorted(list(sta["detail"][self.categories[0]]["ratio_distribution"].keys()))
            for r in cat_ratios:
                cats_detail_table_data.append(
                    [r] + [sta["detail"][cat]["ratio_distribution"].get(r, 0) for cat in self.categories])

            cats_detail_table = AsciiTable(cats_detail_table_data)
            cats_detail_table.inner_footing_row_border = False

            print("overview[概览]:")
            print(overview_table.table)

            print("\nclass detail[类别数量分布]:")
            print(cats_detail_table.table)

            print("\ntarget statistic[目标框/分割区域统计]:")
            print(target_statistic_table.table)

            print("\nbbox ratio (h/w) distribution[目标框比例(高/宽)分布]:")
            print(ratio_distri_table.table)

            if is_plot_show or plot_save_path is not None:
                import matplotlib.pyplot as plt
                fig = plt.figure(num=5, figsize=(15, 8), dpi=100)

                ax1 = fig.add_subplot(3, 2, 1)
                overview_header_post = overview_header[:3] + overview_header[4:]
                overview_table_data_post = overview_table_data[:3] + overview_table_data[4:]
                b1 = ax1.barh(list(range(len(overview_header_post))), overview_table_data_post, color='#6699CC',
                              tick_label=overview_header_post)
                for i, rect in enumerate(b1):
                    w = rect.get_width()
                    ax1.text(w + 1, rect.get_y() + rect.get_height() / 2, '%d' % int(overview_table_data_post[i]),
                             ha='left',
                             va='center')

                ax1.set_title("overview")

                ax2 = fig.add_subplot(3, 2, 2)
                b2 = ax2.bar(list(range(len(self.categories))), cats_num_data, color="red", tick_label=self.categories)
                for xi, rect in enumerate(b2):
                    ax2.text(rect.get_x() + rect.get_width() / 2, rect.get_height(), "%d" % int(cats_num_data[xi]),
                             ha="center", va="bottom")

                ax2.set_title("class detail")

                ax3 = fig.add_subplot(3, 2, 3)
                b3 = ax3.barh(list(range(len(target_statistic_header))), target_statistic_data, color="green",
                              tick_label=target_statistic_header)
                for i, rect in enumerate(b3):
                    w = rect.get_width()
                    ax3.text(w, rect.get_y() + rect.get_height() / 2, '%d' % int(target_statistic_data[i]),
                             ha='left',
                             va='center')

                ax3.set_title("target statistic")

                ax4 = fig.add_subplot(3, 2, 4)
                b4 = ax4.bar(list(range(len(ratios))), ratio_distri_data, color="blue", tick_label=ratios)
                for xi, rect in enumerate(b4):
                    ax4.text(rect.get_x() + rect.get_width() / 2, rect.get_height(), "%d" % int(ratio_distri_data[xi]),
                             ha="center", va="bottom")

                ax4.set_title("bbox ratio (h/w) distribution")

                ax5 = fig.add_subplot(3, 2, 5)
                for cat in bbox_size_dict:
                    w, h = zip(*bbox_size_dict[cat])
                    ax5.scatter(w, h, label=cat)

                ax5.set_xlabel("bbox width")
                ax5.set_ylabel("bbox height")
                ax5.set_title("bbox size distribution")
                ax5.legend()

                if is_plot_show:
                    fig.show()

                if plot_save_path is not None:
                    fig.savefig(plot_save_path)

        return sta



from .coco import Coco
from .voc import Voc
from .labelme import LabelMe
from .elf import Elf
from .dataset import IcvDataSet
from .classify import Classify
from .shape import bbox_scaling, bbox_clip, bbox_extend
from .converter import VocConverter, CocoConverter, LabelMeConverter, ElfConverter

__all__ = ['Coco', 'Voc', 'LabelMe', 'Elf', 'IcvDataSet', 'Classify', 'bbox_scaling', 'bbox_clip', 'bbox_extend',
           'VocConverter', 'CocoConverter', 'LabelMeConverter', 'ElfConverter']



# -*- coding: UTF-8 -*-
import os
import json
import random
import numpy as np
from skimage import measure
from tqdm import tqdm
import shutil
from .dataset import IcvDataSet
from ..data.core.bbox import BBox
from ..data.core.mask import Mask
from ..data.core.sample import Sample, Anno
from ..data.core.meta import AnnoMeta,SampleMeta
from ..vis.color import VIS_COLOR
from icv.image import imread
from icv.utils import is_file,is_seq,is_dict,xml2json,mkdir

class ElfFiles(object):
    def __init__(self,image_file,anno_file,attachment_dir=None):
        assert is_file(image_file)
        assert is_file(anno_file)
        assert attachment_dir is None or is_file(attachment_dir)

        self.image_file = image_file
        self.anno_file = anno_file
        self.attachment_file = attachment_dir

# TODO: bug 分割的label解析的有问题
class Elf(IcvDataSet):
    def __init__(self,image_anno_attachment_pathlist, split="trainval", keep_no_anno_image=True, categories=None,
                 one_index=False):
        assert is_seq(image_anno_attachment_pathlist)

        image_path_list, anno_path_list, attachment_dir_list = [],[], []
        for f in image_anno_attachment_pathlist:
            assert is_seq(f) or is_dict(f) or isinstance(f,ElfFiles)
            if is_seq(f):
                assert len(f) >= 2
                image_path_list.append(f[0])
                anno_path_list.append(f[1])

                if len(f) > 2:
                    attachment_dir_list.append(f[2])
                else:
                    attachment_dir_list.append(None)
            elif is_dict(f):
                assert "image_file" in f and "anno_file" in f
                image_path_list.append(f["image_file"])
                anno_path_list.append(f["anno_file"])

                if "attachment_dir" in f:
                    attachment_dir_list.append(f["attachment_dir"])
                else:
                    attachment_dir_list.append(None)
            else:
                image_path_list.append(f.image_file)
                anno_path_list.append(f.anno_file)
                attachment_dir_list.append(f.attachment_file)

        self.split = split
        self.keep_no_anno_image = keep_no_anno_image
        self.one_index = one_index

        self.ids = [os.path.basename(_).rsplit(".", 1)[0] for _ in image_path_list]
        self.id2imgpath = {id: image_path_list[ix] for ix, id in enumerate(self.ids)}
        self.id2annopath = {id: anno_path_list[ix] for ix, id in enumerate(self.ids)}
        self.id2attachpath = {id: attachment_dir_list[ix] for ix, id in enumerate(self.ids)}

        self.sample_db = {}
        self.color_map = {}
        self.get_samples()
        self.categories = categories if categories is not None else self.parse_categories()

        super(Elf, self).__init__(self.ids, self.categories, self.keep_no_anno_image, one_index)
        print("there have %d samples in LabelMe dataset" % len(self.ids))
        print("there have %d categories in LabelMe dataset" % len(self.categories))

    def _get_bboxes_from_mask(self,mask_or_path):
        mask = imread(mask_or_path,0)
        contours = measure.find_contours(mask, 0.5)

        reshaped_contour = []
        for contour in contours:
            contour = np.flip(contour, axis=1)
            s = contour.ravel().tolist()
            assert len(s) % 2 == 0
            s = [(s[i], s[i + 1]) for i in range(len(s)) if i % 2 == 0]
            reshaped_contour.append(s)

        bboxes = []
        for i in range(len(reshaped_contour)):
            contour = np.array(reshaped_contour[i])
            xmin,xmax = int(np.min(contour[:,0])),int(np.max(contour[:,0]))
            ymin,ymax = int(np.min(contour[:,1])),int(np.max(contour[:,1]))
            bboxes.append([xmin,ymin,xmax,ymax])

        return bboxes

    def _parse_anno_item(self,ix,obj,id,anno_data):
        if "name" in obj:
            label = obj["name"]
            if label not in self.color_map:
                self.color_map[label] = random.choice(VIS_COLOR)
            if "bndbox" in obj:
                xmin = obj["bndbox"]["xmin"]
                ymin = obj["bndbox"]["ymin"]
                xmax = obj["bndbox"]["xmax"]
                ymax = obj["bndbox"]["ymax"]

                anno = Anno(
                    bbox=BBox(xmin=xmin, ymin=ymin, xmax=xmax, ymax=ymax, label=label),
                    label=label,
                    color=self.color_map[label],
                    meta=AnnoMeta()
                )

                return [anno]
            elif "attachments" in anno_data or ("doc" in anno_data and "attachments" in anno_data["doc"]): # for segmentation
                attachments = anno_data["attachments"] if "attachments" in anno_data else anno_data["doc"]["attachments"]
                def _parse_anno_from_attach(attach):
                    annos = []
                    if attach is not None and "file_name" in attach:
                        mask_path = os.path.join(self.id2attachpath[id],attach["file_name"])
                        img_mask = imread(mask_path,0)
                        bboxes = self._get_bboxes_from_mask(img_mask)
                        for bbox in bboxes:
                            in_others = False
                            for _bbox in bboxes:
                                if BBox.init_from(bbox).is_in(_bbox):
                                    in_others = True
                                    break
                            if in_others:
                                continue

                            xmin,ymin,xmax,ymax = bbox
                            anno = Anno(
                                bbox=BBox(xmin=xmin, ymin=ymin, xmax=xmax, ymax=ymax, label=label),
                                label=label,
                                color=self.color_map[label],
                                meta=AnnoMeta()
                            )

                            bin_mask = np.zeros_like(img_mask, np.uint8)
                            bin_mask[ymin:ymax, xmin:xmax] = img_mask[ymin:ymax, xmin:xmax]
                            bin_mask[np.where(bin_mask != 0)] = 1
                            anno.mask = Mask(bin_mask)

                            annos.append(anno)

                    return annos

                if is_seq(attachments):
                    return _parse_anno_from_attach(attachments[ix])
                elif is_dict(attachments) and "attachment" in attachments:
                    attachment = attachments["attachment"]
                    if is_seq(attachment):
                        return _parse_anno_from_attach(attachment[ix])
                    return _parse_anno_from_attach(attachment)

        return None

    def _parse_json_annotation(self,id,anno_data):
        annos = []
        if "labeled" in anno_data and not anno_data["labeled"]:
            return annos

        if "outputs" in anno_data and "object" in anno_data["outputs"]:
            for ix,obj in enumerate(anno_data["outputs"]["object"]):
                anno = self._parse_anno_item(ix,obj,id,anno_data)
                if anno is not None:
                    annos.extend(anno)

        return annos

    def _parse_xml_annotation(self,id,anno_data):
        annos = []
        if "labeled" in anno_data and not anno_data["labeled"]:
            return annos

        if "annotation" in anno_data:   # for pascal-voc (only detection)
            if "object" in anno_data["annotation"]:
                object = anno_data["annotation"]["object"]
                if not is_seq(object):
                    object = [object]
                for obj in object:
                    anno = self._parse_anno_item(0,obj,id,anno_data)
                    if anno is not None:
                        annos.extend(anno)
        elif "doc" in anno_data:    # for xml style (detection or segmentation)
            if "outputs" in anno_data["doc"] and "object" in anno_data["doc"]["outputs"] and "item" in anno_data["doc"]["outputs"]["object"]:
                items = anno_data["doc"]["outputs"]["object"]["item"]
                if not is_seq(items):
                    items = [items]
                for ix,obj in enumerate(items):
                    anno = self._parse_anno_item(ix,obj,id,anno_data)
                    if anno is not None:
                        annos.extend(anno)

        return annos

    def _load_annotation(self,id,anno_file):
        if str(anno_file).endswith(".json"):
            anno_data = json.load(open(anno_file, "r"))
            return self._parse_json_annotation(id,anno_data)
        elif str(anno_file).endswith(".xml"):
            anno_data = xml2json(anno_file)
            return self._parse_xml_annotation(id,anno_data)

    def get_sample(self, id):
        """
        get sample
        :param id: image name
        :return:
        """
        if id in self.sample_db:
            return self.sample_db[id]

        anno_file = self.id2annopath[id]
        annos = self._load_annotation(id,anno_file)
        img_file = self.id2imgpath[id]

        sample = Sample(
            name=id,
            image=img_file,
            annos=annos,
            meta=SampleMeta()
        )
        self.sample_db[id] = sample
        return sample

    def vis(self, id=None, with_bbox=True, with_seg=True, is_show=False, save_dir=None, reset_dir=False):
        if save_dir is not None:
            if not os.path.exists(save_dir):
                mkdir(save_dir)
            elif reset_dir:
                shutil.rmtree(save_dir)
                mkdir(save_dir)

        if id is not None:
            sample = self.get_sample(id)
            save_path = None if save_dir is None else os.path.join(save_dir, "%s.jpg" % sample.name)
            return sample.vis(with_bbox=with_bbox, with_seg=with_seg, is_show=is_show, save_path=save_path)

        image_vis = []
        for id in tqdm(self.ids):
            sample = self.get_sample(id)
            save_path = None if save_dir is None else os.path.join(save_dir, "%s.jpg" % sample.name)
            image = sample.vis(with_bbox=with_bbox, with_seg=with_seg, is_show=is_show, save_path=save_path)
            image_vis.append(image)
        return image_vis


# -*- coding: UTF-8 -*-
import os
import json
import itertools
from .itis import is_dict,is_seq
from xml.etree import ElementTree as etree
from xmljson import badgerfish as bf

def check_file_exist(filename, msg_tmpl='file "{}" does not exist'):
    if not os.path.isfile(filename):
        raise FileNotFoundError(msg_tmpl.format(filename))

def concat_list(in_list):
    """Concatenate a list of list into a single list.

    Args:
        in_list (list): The list of list to be merged.

    Returns:
        list: The concatenated flat list.
    """
    return list(itertools.chain(*in_list))

def xml2json(xml_path_or_content):
    if str(xml_path_or_content).endswith(".xml"):
        with open(xml_path_or_content, 'r') as fid:
            xml_str = fid.read()
    else:
        xml_str = xml_path_or_content
    bf_data = bf.data(etree.fromstring(xml_str))
    bf_data = json.loads(json.dumps(bf_data))

    def _format(data):
        if is_dict(data):
            for k in data:
                if k == '$':
                    return data[k]
                data[k] = _format(data[k])
        elif is_seq(data):
            for i in range(len(data)):
                data[i] = _format(data[i])
        return data

    bf_data = _format(bf_data)
    return bf_data

